import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:uuid/uuid.dart';

import 'extended_getx_controller.dart';
import 'status_controller.dart';

class BaseController extends ExtendedGetxController {
  final _status = StatusController();

  List<StreamSubscription> _streams = [];

  void autoDispose(StreamSubscription stream) => _streams.add(stream);

  @override
  void onClose() {
    _streams.forEach((element) => element.cancel());
    _streams = [];
    super.onClose();
  }

  Future<T?> enqueue<T>(
    Future<T> Function() task, {
    bool ignoreIfLoading = false,
  }) async {
    if (ignoreIfLoading && isProcessing) return null;

    final job = Uuid();
    processes.add(job);
    _status.add(job);

    final result = await task().catchError(
      (error, stackTrace) async {
        _status.catchError(error);
        if (kDebugMode) {
          print(stackTrace);
        }
        return null;
      },
    ).whenComplete(
      () {
        processes.remove(job);
        _status.complete(job);
      },
    );

    return result;
  }

  void catchError(dynamic source) {
    _status.catchError(source);
  }
}
